// All database related API calls

import { useAuth } from '../authorization';
import { useSessionCache } from '../store';

export const useSchemaAPI = () => {
  const cache = useSessionCache();
  const domain = import.meta.env.BACKEND_URL;
  const schema = import.meta.env.BACKEND_SCHEMA;
  const { fetchAuthenticated } = useAuth();

  async function RequestSchema(databaseName?: string) {
    if (!databaseName) databaseName = cache.currentDatabase;
    if (!databaseName) {
      throw Error('Must call with a database name');
    }

    const request = {
      databaseName,
      cached: false,
      // cached: true,
    };

    const response = await fetchAuthenticated(`${domain}${schema}`, {
      method: 'POST',
      body: JSON.stringify(request),
    });

    if (!response?.ok) {
      const ret = await response.text();
      console.error(response, ret);
    }
    // const ret = await response.json();
    // console.debug('Schema Requested', response);
  }

  return { RequestSchema };
};
